/*
   Project: ImpersonatorToolKit

   Copyright (C) 2013 Sebastian Reitenbach

   Author: Sebastian Reitenbach

   Created: 2013-06-24 19:04:21 +0200 by sebastia

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#ifndef _CONNECTIONMANAGER_H_
#define _CONNECTIONMANAGER_H_

#import <Foundation/Foundation.h>
#import "Connection.h"
#import "LinkLayer.h"

@interface ConnectionManager : NSObject
{
  /* The highest offset and the buffer size allocated, and the buffer
   * itself. */
  NSUInteger len;
  NSUInteger alloc;

  /* Flag indicating that we've seen a FIN-flagged segment for this stream,
   * so that it is undergoing a shutdown. */
  NSInteger fin;

  /* The time at which we last received any data on this stream. */
  time_t last;

  NSMutableArray *slots;

  BOOL connectionManagerDebug;
}

- (Connection *) findConnectionForSourceIP: (NSString *)_srcIP
				sourcePort: (NSUInteger)_srcPort
		          andDestinationIP: (NSString *)_dstIP
			   destinationPort: (NSUInteger)_dstPort;
				

- (BOOL) newConnectionForPacket: (NSData *)_packet
		     withLLInfo: (LinkLayer *) _llinfo;

- (void) deleteConnection: (Connection *)_connection;

- (void) sweepConnections;

@end

#endif // _CONNECTIONMANAGER_H_

