/*
   Project: ImpersonatorToolKit

   Copyright (C) 2013 Sebastian Reitenbach

   Author: Sebastian Reitenbach

   Created: 2013-06-24 19:04:21 +0200 by sebastia

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import "ConnectionManager.h"
#import "PayloadManager.h"

/* sweepConnections:
 * Free finished connection slots. */
#define TIMEOUT 5
#define MAXCONNECTIONDATA   (10 * 1024)
#define MINCONNECTIONDATA   15

@implementation ConnectionManager

-(id)init
{
  if ((self = [super init]) != nil)
    {
      NSUserDefaults *ud =[NSUserDefaults standardUserDefaults];
      slots = [[NSMutableArray alloc] init];
      connectionManagerDebug = [ud boolForKey:@"ConnectionManagerDebug"];
    } 
  return self;
}

-(void)dealloc
{
  [slots release];
  [super dealloc];
}

- (Connection *) findConnectionForSourceIP: (NSString *)_srcIP
				sourcePort: (NSUInteger)_srcPort
                          andDestinationIP: (NSString *)_dstIP
			   destinationPort: (NSUInteger)_dstPort
{
  for (id C in slots)
    {
//NSLog(@"comparing srcIP: %@ %@", [C srcIP], _srcIP);
//NSLog(@"comparing dstIP: %@ %@", [C dstIP], _dstIP);
//NSLog(@"comparing srcPort: %"PRIuPTR" %"PRIuPTR, [C srcPort], _srcPort);
//NSLog(@"comparing dstPort: %"PRIuPTR" %"PRIuPTR, [C dstPort], _dstPort);
      if ([[C srcIP] isEqual:_srcIP] && 
	  [[C dstIP] isEqual:_dstIP] &&
	  [C srcPort] == _srcPort &&
	  [C dstPort] == _dstPort)
	{
	  return C;
        }
    }
  return NULL;
}

- (BOOL) newConnectionForPacket: (NSData *)_packet
                     withLLInfo: (LinkLayer *) _llinfo
{
  Connection *C = [[Connection alloc] init];
  NSRange range;
  range.location = [_llinfo offsetNext];
  range.length = [_packet length] - range.location;
  if (range.length)
    {
      // only append data when there is actual payload
      [C appendData: [NSData dataWithData: [_packet subdataWithRange:range]]];
    }

  [C setSrcIP: [_llinfo srcIP]];
  [C setDstIP: [_llinfo dstIP]];
  [C setSrcPort: [_llinfo srcPort]];
  [C setDstPort: [_llinfo dstPort]];
  [C setLast: time(NULL)];
  [slots addObject: C];

  [C release];
  return YES;
}

- (void) deleteConnection: (Connection *)_connection
{
  [slots removeObject:_connection];
}

- (void) sweepConnections
{
  time_t now;
  PayloadManager *payloadManager = [PayloadManager sharedPayloadManager];
  NSArray *slotA = [NSArray arrayWithArray:slots];
  now = time(NULL);
  for (id C in slotA)
    {
      if (now - [C last] > TIMEOUT ||
		[C fin] || [[C data] length] > MAXCONNECTIONDATA)
	{
	  if ([[C data] length] > MINCONNECTIONDATA)
	    {
	      if (connectionManagerDebug)
	        NSLog(@"saving data for connection: %@ -> %@", [C srcIP], [C dstIP]);
	      [payloadManager savePayloadForConnection:C];
	    }
	  else
	    {
	      if (connectionManagerDebug)
	        NSLog(@"not saving data for connection: %@ -> %@", [C srcIP], [C dstIP]);
	    }
	  [self deleteConnection:C];
	  C = NULL;
	}
    }
}

@end
