/*
   Project: ImpersonatorToolKit

   Copyright (C) 2013 Sebastian Reitenbach

   Author: Sebastian Reitenbach

   Created: 2013-06-30 17:09:45 +0200 by sebastia

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#ifndef _DATABASEMANAGER_H_
#define _DATABASEMANAGER_H_

#import <Foundation/Foundation.h>
#import <SQLClient/SQLClient.h>
#import "Payload.h"

@interface DatabaseManager : NSObject
{
  SQLClient *cookieDB;
  NSString *sortRow;
  NSString *sortOrder;
}

+(id) sharedDatabaseManager;

// configure sorting when retrieving data
- (void) setSortRow: (NSString *) _sortrow;
- (void) setSortOrder: (NSString *) _sortorder;

// save data in the database
-(void)saveCookiesToDB:(Payload *)_payload;

// retrieve data from the database
- (NSArray *) getClientIPs;
- (NSArray *) getHostnames;
- (NSArray *) getHostsAccessedbyClient: (NSString *)_clientIP;
- (NSArray *) getClientsAccessingHost: (NSString *)_targetHost;
- (NSArray *) getCookiesForClientIP:(NSString *)_clientIP
		      andTargetHost:(NSString *)_targetHost;
- (NSArray *) getDistinctURLsForClientIP:(NSString *)_clientIP
		           andTargetHost:(NSString *)_targetHost;

- (void) deleteCookiesForClient:(NSString *)_client;
- (void) deleteCookiesForClient:(NSString *)_client
		        andHost:(NSString *)_host;
- (void) deleteCookiesForHost: (NSString *)_host;
@end

#endif // _DATABASEMANAGER_H_

