/*
   Project: ImpersonatorToolKit

   Copyright (C) 2013 Sebastian Reitenbach

   Author: Sebastian Reitenbach

   Created: 2013-06-25 19:11:32 +0200 by sebastia

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#ifndef _LINKLAYER_H_
#define _LINKLAYER_H_

#import <Foundation/Foundation.h>
#import "DataLink.h"
#include <netinet/tcp.h>

#define ETH_ADDRLEN  6 

@interface LinkLayer : NSObject
{
  unsigned char dest[ETH_ADDRLEN];
  unsigned char src[ETH_ADDRLEN];
  u_int16_t     proto;
  NSUInteger    offsetNext;		// offset to the next header on each stage parsing headers
  NSUInteger    offsetData;		// offset to the data from beginning of packet
  NSInteger     nextProto;
  NSString 	*srcIP;			// source IP address (IPv6 or IPv4)
  NSString	*dstIP;			// destination IP address (IPv6 or IPv4)
  NSUInteger    srcPort;		// tcp source port
  NSUInteger    dstPort;		// tcp destination port
  NSUInteger	tcpSequence;		// tcp sequence number
  tcp_seq	tcpAck;			// tcp acknowledge number
  NSUInteger	tcpFlags;		// tcp header flags
}

- (NSUInteger) offsetNext;
- (NSUInteger) offsetData;                             
- (NSInteger) nextProto;                             
- (NSUInteger) srcPort;
- (NSUInteger) dstPort;
- (NSUInteger) tcpSequence;
- (NSUInteger) tcpFlags;
- (NSString *) srcIP;
- (NSString *) dstIP;

- (BOOL) handleLinkLayerForDataLink: (DataLink *)_dataLink
                             packet: (NSData *)_packet;
                             
- (BOOL) findTcpForPacket:(NSData *)_packet;

@end

#endif // _LINKLAYER_H_

