/*
   Project: ImpersonatorToolKit

   Copyright (C) 2013 Sebastian Reitenbach

   Author: Sebastian Reitenbach

   Created: 2013-06-23 15:29:23 +0200 by sebastia

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#ifndef _SNIFFERCONTROLLER_H_
#define _SNIFFERCONTROLLER_H_

#import <Foundation/Foundation.h>
#import <PcapKit/PcapKit.h>
#import "DataLink.h"
#import "ConnectionManager.h"

@interface SnifferController : NSObject <PcapKitDelegate>
{
  PcapKit *sniffer;
  DataLink *dataLink;
  ConnectionManager *connectionManager;
  NSString *inputFile;
  BOOL snifferControllerDebug;
}

// needed setters and getters
- (void) setDevice: (NSString *) _dev;
- (void) setInputFile: (NSString *) _inputFile;
- (NSString *) inputFile;
- (void) setPromiscuous: (BOOL) _promisc;
- (void) setReadTimeout:(NSInteger) _rt;
- (void) setBpfFilter:(NSString *) _bpf;
- (void) setOptimizeFilter:(BOOL) _optimize;
// interact with the pcap sniffer
- (void) openLiveStream;
- (void) openOfflineStream;
// create a sniffing handle to be configured and used later
-(BOOL) createSniffingHandle;
// activate a configured sniffing handle
-(NSInteger) activateSniffingHandle;

- (BOOL) applyFilter;
- (NSInteger) startPcapLoop;
- (void) setDelegate;

// the delegate that handles the packet here
-(void)handlePacket: (NSData *)_packet;


// if no device give, try to find it automatically
- (BOOL) findDeviceAutomatically;
// return list of datalinks in the given array
-(NSInteger) getDatalinksIntoArray:(int **)_dlts;

// test if the device supports RFMon mode
-(BOOL)checkSetRFMon;
// set the device into RFMon mode
-(BOOL)enableRFMon:(BOOL)_flag;

@end

#endif // _SNIFFERCONTROLLER_H_

