/*
   Project: ImpersonatorToolKit

   Copyright (C) 2013 Sebastian Reitenbach

   Author: Sebastian Reitenbach

   Created: 2013-06-23 15:28:47 +0200 by sebastia

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import "SnifferController.h"
#import "LinkLayer.h"

@implementation SnifferController

-(id)init
{
  if ((self = [super init]) != nil)
    {
      NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
      inputFile = nil;
      sniffer = [[PcapKit alloc] init];
      connectionManager = [[ConnectionManager alloc] init];
      snifferControllerDebug = [ud boolForKey:@"SnifferControllerDebug"];
    }
  return self;
}

-(void) dealloc
{
  [sniffer release];
  [connectionManager release];
  [dataLink release];
  [inputFile release];
  [super dealloc];
}

- (void) setDevice: (NSString *) _dev
{
  [sniffer setDevice:_dev];
}
- (void) setInputFile: (NSString *) _inputFile
{
  ASSIGN(inputFile,_inputFile);
  [sniffer setInputFile: _inputFile];
}
- (NSString *)inputFile
{
  return inputFile;
}
- (void) setPromiscuous: (BOOL) _promisc
{
  [sniffer setPromisc: _promisc];
}
- (void) setReadTimeout:(NSInteger) _rt
{
  [sniffer setReadTimeout: _rt];
}
- (void) setBpfFilter:(NSString *) _bpf
{
  [sniffer setBpfFilter:_bpf];
}
- (void) setOptimizeFilter:(BOOL) _optimize
{
  [sniffer setOptimizeFilter:_optimize];
}
-(void) openLiveStream
{
  [sniffer openLiveStream];
  dataLink = [[DataLink alloc] initWithSniffer: sniffer];
}
-(void) openOfflineStream
{
  [sniffer openOfflineStream];
  dataLink = [[DataLink alloc] initWithSniffer: sniffer];
}
-(BOOL) applyFilter
{
  return [sniffer applyFilter];
}
-(NSInteger) startPcapLoop
{
  return [sniffer startPcapLoop];
}
-(void) setDelegate
{
  [sniffer setDelegate:self];
}

- (BOOL) findDeviceAutomatically
{
  return [sniffer lookupDevice];
}

// the PcapKit delegate that handles the packet here
-(void)handlePacket: (NSData *)_packet
{
  static NSUInteger count = 1;	// the packet counter
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

  /* declare pointers to packet headers */
  LinkLayer *linkLayer;
  Connection *connection; 
  
  linkLayer = [[LinkLayer alloc] init];
  
  count ++;

//sleep(3);
  if (![linkLayer handleLinkLayerForDataLink:dataLink packet:_packet])
    {
      [_packet release];
      [linkLayer release];
      [pool release];
      return;
    }
  
  if (![linkLayer findTcpForPacket:_packet])
    {
      [_packet release];
      [linkLayer release];
      [pool release];
      return;
    }

  connection = [connectionManager findConnectionForSourceIP:
					[linkLayer srcIP]
						 sourcePort:
					[linkLayer srcPort]
					   andDestinationIP:
					[linkLayer dstIP]
					    destinationPort:
					[linkLayer dstPort]];
  
  if (!connection && [linkLayer tcpFlags] & TH_SYN)
	// it must be a new unknown connection
    {
      if (snifferControllerDebug)
        NSLog(@"new connection from: %@ -> %@", [linkLayer srcIP],
						[linkLayer dstIP]);
      [connectionManager newConnectionForPacket:_packet withLLInfo: linkLayer];
      // with the link layer released here, sniffer will crash
    }

  if ([linkLayer tcpFlags] & TH_RST || [linkLayer tcpFlags] & TH_FIN)
    {
      // the sweep at the end of the method will delete packets which
      //have fin set to 1
      [connection setFin:1];
      // since we have two connections per stream, have to remove the
      // other half of it too
      connection = nil;
      connection = [connectionManager findConnectionForSourceIP:
						[[linkLayer dstIP] retain]
						     sourcePort:
						[linkLayer dstPort]
					       andDestinationIP:
						[[linkLayer srcIP] retain]
						destinationPort:
						[linkLayer srcPort]];
      [connection setFin:1];
      if (snifferControllerDebug)
	{
          NSLog(@"marking connection as terminated: %@ -> %@", 
				[linkLayer srcIP], [linkLayer dstIP]);
          NSLog(@"marking connection as terminated: %@ -> %@",
				[linkLayer dstIP], [linkLayer srcIP]);
	}
      // commenting the following _packet release will make the tool
      // not crashing on exit, see below too
      [_packet release];
      [linkLayer release];
      [pool release];
      return;
    }

  NSRange range;
#warning should check for out of order packets!!!
  range.location = [linkLayer offsetNext];
  range.length = [_packet length] - [linkLayer offsetNext];
  connection = NULL;
  connection = [connectionManager findConnectionForSourceIP:
					[[linkLayer srcIP] retain]
						 sourcePort:
					[linkLayer srcPort]
					   andDestinationIP:
					[[linkLayer dstIP] retain]
					    destinationPort:
					[linkLayer dstPort]];
  if (range.length && connection)
    {
      // only append data when there is payload
      NSData *data = [NSData dataWithData:[_packet subdataWithRange:range]];
      if (snifferControllerDebug)
        NSLog(@"adding data for connection: %@ -> %@",
			[linkLayer srcIP], [linkLayer dstIP]);
      [connection appendData: data];
    }
  else
    {
      if (snifferControllerDebug)
        NSLog(@"skipping packet for connection: %@ -> %@",
			[linkLayer srcIP], [linkLayer dstIP]);
    }
  // commenting the following _packet release will make the tool
  // not crashing on exit, see above too
  [_packet release];
  [linkLayer release];
  [connectionManager sweepConnections];
  [pool release];
}

-(BOOL) createSniffingHandle
{
  return [sniffer createSniffingHandle];
}
-(NSInteger) activateSniffingHandle
{
  return [sniffer activateSniffingHandle];
}

-(BOOL)checkSetRFMon
{
  return [sniffer checkSetRFMon];
}
-(BOOL)enableRFMon:(BOOL)_flag
{
  [sniffer enableRFMon:_flag];
  return YES;
}

-(NSInteger) getDatalinksIntoArray:(int **)_dlts
{
  return [sniffer getDatalinksIntoArray:_dlts];
}

@end
