/*
   Project: ImpersonatorToolKit

   Copyright (C) 2013 Sebastian Reitenbach

   Author: Sebastian Reitenbach

   Created: 2013-06-30 15:35:48 +0200 by sebastia

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#ifndef _WEBSERVERHANDLER_H_
#define _WEBSERVERHANDLER_H_

#import <Foundation/Foundation.h>
#import <GNUstepBase/GSMime.h>
#import <WebServer/WebServer.h>
#import "DatabaseManager.h"

@interface WebServerHandler : NSObject <WebServerDelegate>
{
  DatabaseManager *dbManager;
  WebServer *webServer;
  NSString *serverPort;
  NSArray *clientIPs;		// the list of unique client IPs in the database
  NSArray *hosts;		// the hosts where we have cookies for a given clientIP
  NSString *clientIP;		// the client IP where we request the cookies for
  NSString *host;		// the host, of which we are requested to set cookies for
  NSArray *urls;		// URLs on that host the sniffer has logged
  NSString *httpPath;		// the query where the user clicked on the generated web page
  NSArray *cookies;		// list of cookies for the given host and client IP and other infos about them
  NSURL *referer;		// where does the request come from
  BOOL webServerHandlerDebug;	// shall we print out debug outut?
}

- (void)runWebserver;

- (void)setWebServerPort:(NSString *)_serverPort;

// the delegate methods for the WebServerDelegate protocol
- (BOOL) processRequest: (GSMimeDocument*)request
               response: (GSMimeDocument*)response
                    for: (WebServer*)http;
- (void) webAlert: (NSString*)message for: (WebServer*)http;
- (void) webAudit: (NSString*)message for: (WebServer*)http;
- (void) webLog: (NSString*)message for: (WebServer*)http;

@end

#endif // _WEBSERVERHANDLER_H_

