/*
   Project: ImpersonatorToolKit

   Copyright (C) 2013 Sebastian Reitenbach

   Author: Sebastian Reitenbach

   Created: 2013-06-30 15:35:48 +0200 by sebastia

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import "WebServerHandler.h"
#import "DatabaseManager.h"

static NSString *HTTP_COOKIE_DEFAULT_EXPIRES =
			@"Tuesday, 2-Feb-2020 02:02:02 GMT";

static NSString *PAGE_HEAD = @"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
        @"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" "
        @"\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"
        @"<html xmlns=\"http://www.w3.org/1999/xhtml\">\n"
        @"<head><title>Impersonator Proxy</title>\n"
        @"<link rel=\"stylesheet\" type=\"text/css\""
        @"href=\"/impersonator.css\"></head>\n";

@interface WebServerHandler(Private)
-(NSString *)createMainPage;
-(NSString *)createMainPageForClients;
-(NSString *)createPageForClientIP:(NSString *)_clientIP;
-(NSString *)createPageForClientIP:(NSString *)_clientIP 
			   andHost:(NSString *)_host;
-(NSString *)createMainPageForHosts;
-(NSString *)createPageForDestinationHost:(NSString *)_destinationHostname;
-(NSString *)createPageForDestinationHost:(NSString *)_destinationHostname 
			   andClientIP:(NSString *)_clientIP;
@end

@implementation WebServerHandler(Private)
-(NSString *)createMainPage
{
  NSMutableString *pageContents;
  
  pageContents = [NSMutableString stringWithFormat:@"%@", PAGE_HEAD];
  [pageContents appendString:
		@"<body><table>\n"
		@"<thead><tr><th class=\"hover\"><a href=\"/src\">"
		@"Client search</a></th>\n"
		@"<th class=\"hover\"><a href=\"/dst\">"
		@"Server search</a></th></tr>\n"
		@"</thead></table></body></html>\n"];

  return pageContents;

}

-(NSString *)createMainPageForClients
{
  NSMutableString *pageContents;
  clientIPs = [dbManager getClientIPs];

  pageContents = [NSMutableString stringWithFormat:@"%@", PAGE_HEAD];
  [pageContents appendString:
		@"<body><table>"
		@"<thead>\n<tr><th class=\"hover\">"
  		@"<a href=\"/\">go to main page</a></th>"
		@"</tr>\n</thead>\n</table>\n"];
  [pageContents appendString:[NSString stringWithFormat: 
	@"<div id=\"textbox\">"
	@"which client you want to impersonate?<br/></div>\n"
	@"<table>\n"
	@"<thead><tr><th class=\"hover\">"
	@"<a href=\"/src/?action=sortTable&sortrow=clients\">Clients</a></th>\n"
	@"<th class=\"hover\"><a href=\"/src/?action=sortTable&sortrow=count\">"
	@"Nr. of servers</a></th>\n"
	@"<th class=\"hover\">"
	@"<a href=\"/src/?action=sortTable&sortrow=timestamp\">"
	@"Time</a></th>\n"
	@"<th>Action</th>\n"
	@"</tr></thead><tbody>\n"]];

  for (id IPa in clientIPs)
    {
      NSString *IP = [IPa objectAtIndex:0];
      NSString *hostCount = [IPa objectAtIndex:1];
      NSString *ts = [IPa objectAtIndex:2];
      [pageContents appendString:
	[NSString stringWithFormat:@"<tr><td><a href=\"/src/%@\">%@</a></td>"
		@"<td>%@</td>"
		@"<td>%@</td>"
		@"<td><a href=\"/src/?action=deleteDBCookies&sourceclient=%@\">"
		@"delete</a>"
		@"</td></tr>\n", IP, IP, hostCount, ts, IP]];
    }
  [pageContents appendString:@"</tbody></table></body></html>\n"];
  return pageContents;
}

-(NSString *)createMainPageForHosts
{
  NSMutableString *pageContents;
  hosts = [dbManager getHostnames];

  pageContents = [NSMutableString stringWithFormat:@"%@", PAGE_HEAD];
  [pageContents appendString:
		@"<body>\n<table>"
		@"<thead>\n<tr><th class=\"hover\">"
  		@"<a href=\"/\">go to main page</a></th>"
		@"</tr></thead></table>\n"];
  [pageContents appendString: @"<div id=\"textbox\">"
		@"on which host you want to impersonate?</div>\n"
		@"<table id=\"maintable\">\n"
		@"<thead><tr><th class=\"hover\">"
		@"<a href=\"/dst/?action=sortTable&sortrow=server\">"
		@"Server</a></th class=\"hover\">\n"
		@"<th class=\"hover\">"
		@"<a href=\"/dst/?action=sortTable&sortrow=count\">"
		@"Nr. of clients</a></th>\n"
		@"<th class=\"hover\">"
		@"<a href=\"/dst/?action=sortTable&sortrow=timestamp\">"
		@"Time</a></th>\n"
		@"<th>Action</th></tr></thead><tbody>\n"];

  for (NSArray *hostA in hosts)
    {
      NSString *Host = [hostA objectAtIndex:0];
      NSString *clientCount = [hostA objectAtIndex:2];
      NSString *ts = [hostA objectAtIndex:3];
      [pageContents appendString:
	[NSString stringWithFormat:@"<tr><td><a href=\"/dst/%@\">%@</a></td>"
		@"<td>%@</td>"
		@"<td>%@</td><td>"
		@"<a href=\"/dst/?action=deleteDBCookies&destinationhost=%@\">"
		@"delete</a>"
		@"</td></tr>\n", Host, Host, clientCount, ts, Host]];
    }
  [pageContents appendString:@"</tbody></table></body></html>\n"];
  return pageContents;
}

-(NSString *)createPageForClientIP:(NSString *)_clientIP
{
  NSMutableString *pageContents;
  hosts = [dbManager getHostsAccessedbyClient:_clientIP];
  pageContents = [NSMutableString stringWithFormat: @"%@", PAGE_HEAD];
  [pageContents appendString:
	@"<body><table>"
	@"<thead><tr><th class=\"hover\">"
  	@"<a href=\"/\">go to main page</a></th>\n"
	@"<th class=\"hover\"><a href=\"/src\">one level up</a></th>\n"
	@"</tr></thead></table>\n"];
  [pageContents appendString:
	[NSString stringWithFormat:@"<div id=\"textbox\">"
		@"impersonate %@ on one of the targets below<br/></div>\n"
		@"<table id=\"maintable\">"
		@"<thead><tr><th class=\"hover\">"
		@"<a href=/src/%@/?action=sortTable&sortrow=server\">"
		@"Server</a></th>\n"
		@"<th class=\"hover\">"
		@"<a href=/src/%@/?action=sortTable&sortrow=domain\">"
		@"Domain</a></th>\n"
		@"<th class=\"hover\">"
		@"<a href=\"/src/%@/?action=sortTable&sortrow=timestamp\">"
		@"Time</a></th>\n"
		@"<th>Action</th></tr></thead>\n"
		@"<tbody>", _clientIP, _clientIP, _clientIP, _clientIP]];
  for (id Ha in hosts)
    {
      NSString *H 	= [Ha objectAtIndex:0];
      NSString *domain 	= [Ha objectAtIndex:1];
      NSString *ts 	= [Ha objectAtIndex:2];
      [pageContents appendString:[NSString stringWithFormat:
	@"<tr><td><a href=\"http://%@/src/%@/%@\">%@</a></td>"
	@"<td>%@</td>"
	@"<td>%@</td>"
	@"<td><a href=\"/src/%@/?action=deleteDBCookies&"
	@"destinationhost=%@&sourceclient=%@\">delete</a>"
	@"</td></tr>\n",
	H, _clientIP, H, H, domain, ts, _clientIP, H, _clientIP]];
    }
  [pageContents appendString:@"</tbody></table></body></html>\n"];
  return pageContents;
}
-(NSString *)createPageForDestinationHost:(NSString *)_destinationHostname
{
  NSMutableString *pageContents;
  NSArray *clients = [dbManager getClientsAccessingHost:_destinationHostname];
  pageContents = [NSMutableString stringWithFormat:@"%@", PAGE_HEAD];
  [pageContents appendString:@"<body><table id=\"navtable\"><thead>"
			@"<tr><th class=\"hover\">"
  			@"<a href=\"/\">go to main page</a></th>"
			@"<th class=\"hover\">"
			@"<a href=\"/dst\">go one level up</a></th>"
			@"</tr></thead></table>\n"];
  [pageContents appendString:
	[NSString stringWithFormat:@"<div id=\"textbox\">"
		@"select client to impersonate on %@</div>\n"
		@"<table id=\"maintable\">\n"
		@"<thead><tr><th class=\"hover\">"
		@"<a href=\"/dst/%@/?action=sortTable&sortrow=client\">"
		@"Client</a></th class=\"hover\">"
		@"<th class=\"hover\">"
		@"<a href=\"/dst/%@/?action=sortTable&sortrow=timestamp\">"
		@"Time</a></th class=\"hover\">"
		@"<th>Action</th></tr></thead>\n<tbody>\n",
		_destinationHostname, _destinationHostname,
		_destinationHostname]];

  for (NSArray *clientsA in clients)
    {
      NSString *C = [clientsA objectAtIndex:0];
      NSString *ts = [clientsA objectAtIndex:1];
      [pageContents appendString:[NSString stringWithFormat:
	@"<tr><td><a href=\"http://%@/dst/%@/%@\">%@</a></td>\n"
	@"<td>%@</td>\n"
	@"<td><a href=\"/dst/%@/?action=deleteDBCookies&"
	@"destinationhost=%@&sourceclient=%@\">delete</a>"
	@"</td></tr>\n", _destinationHostname, _destinationHostname, C, C,
		ts, _destinationHostname, _destinationHostname, C]];
    }
  [pageContents appendString:@"</tbody>\n</table>\n</body>\n</html>"];
  return pageContents;
}


-(NSString *)createPageForClientIP:(NSString *)_clientIP 
			   andHost:(NSString *)_host
{
  NSMutableString *pageContents;
  cookies = [dbManager getCookiesForClientIP:_clientIP andTargetHost:_host];
  urls = [dbManager getDistinctURLsForClientIP: _clientIP andTargetHost:_host];

  pageContents = [NSMutableString stringWithFormat:@"%@", PAGE_HEAD];
  [pageContents appendString: @"<body><table><thead><tr><th class=\"hover\">"
  				@"<a href=\"/\">go to main page</a></th>\n"];
  [pageContents appendString: [NSString stringWithFormat:
	@"<th class=\"hover\"><a href=\"/src/%@\">go one level up</a>"
	@"</th></tr></thead></table>\n",_clientIP]];
  [pageContents appendString: [NSString stringWithFormat:
	@"<table><thead><tr><th class=\"hover\">"
	@"<a href=\"http://%@?action=setBrowserCookies&"
	@"sourceclient=%@&destinationhost=%@\">"
	@"set cookies in the browser</a></th></tr></thead></table>\n",
	_host, _clientIP, _host]];
  [pageContents appendString: [NSString stringWithFormat:
	@"<div id=\"textbox\">Impersonate client %@ on "
	@"host <a href=\"http://%@\">%@</a>"
	@" with the cookies available below.</br></div>\n",
		_clientIP, _host, _host]];
  [pageContents appendString: [NSString stringWithFormat:
	@"<table id=\"maintable\"><tbody>"]];

  for (id C in urls)
    {
      [pageContents appendString:[NSString stringWithFormat:
	@"<tr><td title=\"http://%@%@\"><a href=\"http://%@%@\">http://%@%@</a></td></tr>\n", 
	_host,[C objectAtIndex:0], _host,[C objectAtIndex:0], _host, [C objectAtIndex:0]]];
    }
  [pageContents appendString:@"</tbody></table>"
	@"<table id=\"maintable\">"];
  [pageContents appendString:@"<thead><tr><th>Cookie</th><th>Value</th>\n"
	@"<th>Path</th>\n"
	@"<th>User Agent</th></tr></thead>\n<tbody>"];

  for (id C in cookies)
    {
      [pageContents appendString:[NSString stringWithFormat:
	@"<tr><td title=\"%@\">%@</td><td title=\"%@\">%@</td><td title=\"%@\">%@</td><td title=\"%@\">%@</td></tr>\n",
	[C objectAtIndex:0], [C objectAtIndex:0], [C objectAtIndex:1], [C objectAtIndex:1],
	[C objectAtIndex:3], [C objectAtIndex:3], [C objectAtIndex:4], [C objectAtIndex:4]]];
    }
  
  [pageContents appendString:@"</tbody></table></body></html>"];
  return pageContents;
}

-(NSString *)createPageForDestinationHost:(NSString *)_destinationHostname
                           andClientIP:(NSString *)_clientIP
{
  NSMutableString *pageContents;
  cookies = [dbManager getCookiesForClientIP:_clientIP
			       andTargetHost:_destinationHostname];

  pageContents = [NSMutableString stringWithFormat:@"%@", PAGE_HEAD];
  [pageContents appendString: @"<body><table><thead><tr><th class=\"hover\">"
  	@"<a href=\"/\">go to main page</a></th><th class=\"hover\">"];
  [pageContents appendString: [NSString stringWithFormat:
	@"<a href=\"/dst/%@\">go one level up</a></th></tr></tbody></table>",
				_destinationHostname]];
  [pageContents appendString: [NSString stringWithFormat:
	@"<table><thead><tr><th class=\"hover\">"
	@"<a href=\"http://%@?action=setBrowserCookies&"
	@"sourceclient=%@&destinationhost=%@\">"
	@"set cookies in the browser</a></th></tr></thead></table>",
	_destinationHostname, _clientIP, _destinationHostname]];
  [pageContents appendString: [NSString stringWithFormat:
	@"<div id=\"textbox\">impersonate client %@ on host "
	@"<a href=\"http://%@\">%@</a>"
	@" with the cookies available below<br/>",
		 _clientIP, _destinationHostname, _destinationHostname]];
  [pageContents appendString: [NSString stringWithFormat:
	@"links below show what the client visited on this host</div>"
	@"<table><tbody>"]];

  for (id C in cookies)
    {
      [pageContents appendString:[NSString stringWithFormat:
	@"<tr><td title=\"http://%@%@\"><a href=\"http://%@%@\">http://%@%@</a></td></tr>", 
	_destinationHostname,[C objectAtIndex:3],
	_destinationHostname,[C objectAtIndex:3],
	_destinationHostname, [C objectAtIndex:3]]];
    }
  [pageContents appendString:@"</tbody></table>"
	@"<table>"];
  [pageContents appendString:@"<thead><tr><th>Cookie</th><th>Value</th>"
	@"<th>Path</th>"
	@"<th>User Agent</th></tr></thead><tbody>"];

  for (id C in cookies)
    {
      [pageContents appendString:[NSString stringWithFormat:
	@"<tr><td title=\"%@\">%@</td><td title=\"%@\">%@</td><td title=\"%@\">%@</td><td title=\"%@\">%@</td></tr>", 
	[C objectAtIndex:0], [C objectAtIndex:0], [C objectAtIndex:1], [C objectAtIndex:1],
	[C objectAtIndex:3], [C objectAtIndex:3], [C objectAtIndex:4], [C objectAtIndex:4]]];
    }
  
  [pageContents appendString:@"</tbody></table></body></html>"];
  return pageContents;
}

@end

@implementation WebServerHandler

-(id)init
{
  NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
  self = [super init];
  if (self == nil)
    return nil;

  dbManager = [DatabaseManager sharedDatabaseManager]; 
  webServerHandlerDebug = [ud boolForKey:@"WebServerHandlerDebug"];
  
  return self;
}

- (void)runWebserver
{
  NSAutoreleasePool *pool;
  pool = [NSAutoreleasePool new];
  webServer = [[WebServer alloc] initForThread:[NSThread currentThread]];
  [webServer setPort:serverPort secure:nil];
  if (webServerHandlerDebug)
    [webServer setVerbose: YES];
  [webServer setDelegate: self];
  [[NSRunLoop currentRunLoop] run];
  [pool release];
}

- (void)setWebServerPort:(NSString *)_serverPort
{
  ASSIGN(serverPort,_serverPort);
}

// the delegate methods for the WebServerDelegate protocol
- (BOOL) processRequest: (GSMimeDocument*)request
               response: (GSMimeDocument*)response
                    for: (WebServer*)http
{
  NSArray *queryPathComponents;

  host = [[request headerNamed:@"host"] value];
  NSArray *refererPathComponents = nil;
  NSString *refString = [[request headerNamed:@"referer"] value];
  NSString *httpQuery;
  NSURL *requestURL;
  NSString *requestQuery;
  NSString *requestPath;

  if (refString != nil)
    {
      referer = [NSURL URLWithString:refString];
      refererPathComponents = [referer pathComponents];
    }
  
  requestURL = [NSURL URLWithString:
		[[request headerNamed:@"x-http-path"] value]];
  requestPath = [requestURL path];
  requestQuery = [[request headerNamed:@"x-http-query"] value];

  httpQuery = [[request headerNamed:@"x-http-query"] value];

  if (![requestQuery isEqual:@""])
    {
      NSString *client = nil;
      NSString *hostname =nil;
      NSString *action = nil;
      NSString *sortrow = nil;
      
       NSArray *queryComponents =
			[httpQuery componentsSeparatedByCharactersInSet:
         [NSCharacterSet characterSetWithCharactersInString:@"&"]];
      for (id queryComponent in queryComponents)
	{
	  NSArray *CP = [queryComponent componentsSeparatedByCharactersInSet:
		[NSCharacterSet characterSetWithCharactersInString:@"="]];
	  if ([[CP objectAtIndex:0] isEqual:@"destinationhost"])
	    {
	      hostname=[CP objectAtIndex:1];
	      continue;
	    }
	  if ([[CP objectAtIndex:0] isEqual:@"sourceclient"])
	    {
	      client = [CP objectAtIndex:1];
	      continue;
	    }
	  if ([[CP objectAtIndex:0] isEqual:@"action"])
	    {
	      action = [CP objectAtIndex:1];
	      continue;
	    }
	  if ([[CP objectAtIndex:0] isEqual:@"sortrow"])
	    {
	      sortrow = [CP objectAtIndex:1];
	      continue;
	    }
	}
      // we're only gonna going to do additional things when we were told so
      if (action != nil)
	{
	  if ([action isEqual:@"setBrowserCookies"])
	    {
	      cookies = [dbManager getCookiesForClientIP:client
					   andTargetHost:hostname];
	      for (id C in cookies)
		{
		  [response addHeader:@"Set-Cookie"
                        value:[NSString stringWithFormat:
                                @"%@=%@",
                                [C objectAtIndex:0], [C objectAtIndex:1]]
                   parameters:[NSDictionary dictionaryWithObjectsAndKeys:
                  @"/", @"Path", HTTP_COOKIE_DEFAULT_EXPIRES, @"Expires", nil]];
	        }
	    }
	  else if ([action isEqual:@"deleteDBCookies"])
	    {
              if (client != nil && hostname == nil)
	        {
	           [dbManager deleteCookiesForClient:client];
	        }
              else if (client == nil && hostname != nil)
	        {
	           [dbManager deleteCookiesForHost:hostname];
	        }
              else if (client != nil && hostname != nil)
	        {
	           [dbManager deleteCookiesForClient:client andHost:hostname];
	        }
	    }
	  else if ([action isEqual:@"sortTable"])
	    {
	      if ([sortrow isEqual:@"clients"])
		{
		  [dbManager setSortRow:@"clientHost"];
		}
	      else if ([sortrow isEqual:@"domain"])
		{
		  [dbManager setSortRow:@"baseDomain"];
		}
	      else if ([sortrow isEqual:@"server"])
		{
		  [dbManager setSortRow:@"host"];
		}
	      else if ([sortrow isEqual:@"timestamp"])
		{
		  [dbManager setSortRow:@"timestamp"];
		}
	      else if ([sortrow isEqual:@"count"])
		{
		  [dbManager setSortRow:@"count"];
		}
	    }
	}
    }

  if ([requestPath isEqual:@"/"])
    {
      NSString *dsthost;
      NSString *clientip;

      if ([refererPathComponents count] < 4)
	{
      	  // user is visiting the main page, so create it
      	  [response setContent:[self createMainPage]
			  type:@"text/html"
			  name:nil];
	}

      // the user may have wanted to set the cookies to the browser, so check
      // the referrer
      if ([[refererPathComponents objectAtIndex:1] isEqual:@"src"])
	{
	  dsthost = [refererPathComponents objectAtIndex:3];
	  clientip = [refererPathComponents objectAtIndex:2];
          [response setContent:[self createPageForClientIP: clientip
		       				   andHost: dsthost]
		          type:@"text/html"
		          name:nil];
	}
      if ([[refererPathComponents objectAtIndex:1] isEqual:@"dst"])
	{
	  dsthost = [refererPathComponents objectAtIndex:2];
	  clientip = [refererPathComponents objectAtIndex:3];
	  [response setContent:[self createPageForDestinationHost: dsthost
						      andClientIP: clientip]
		          type:@"text/html"
		          name:nil];
	}
      return YES;
    }

  queryPathComponents = [requestPath componentsSeparatedByCharactersInSet:
	[NSCharacterSet characterSetWithCharactersInString:@"/"]];

  if ([[queryPathComponents objectAtIndex:1] isEqual:@"src"])
    {
      if ([queryPathComponents count] == 2)	// we only have /src
	{
      	  [response setContent:[self createMainPageForClients]
		      	  type:@"text/html"
		      	  name:nil];
      	  return YES;
	}
      else if ([queryPathComponents count] == 3) // we have /src/IP
	{
      	  [response setContent:[self createPageForClientIP:
	    			[queryPathComponents objectAtIndex:2]]
			  type:@"text/html"
			  name:nil];
          return YES;
	}
      else if ([queryPathComponents count] == 4) // we have /src/IP/dsthost
	{
          [response setContent:[self createPageForClientIP:
				[queryPathComponents objectAtIndex:2]
		       				   andHost:
				[queryPathComponents objectAtIndex:3]]
		          type:@"text/html"
		          name:nil];
          return YES;
	}
    }
  if ([[queryPathComponents objectAtIndex:1] isEqual:@"dst"])
    {
      if ([queryPathComponents count] == 2)     // we only have /dst
	{
          [response setContent:[self createMainPageForHosts]
		          type:@"text/html"
		          name:nil];
          return YES;
	}
      else if ([queryPathComponents count] == 3) // we have /src/dsthost
	{
      	  [response setContent:[self createPageForDestinationHost:
	    			[queryPathComponents objectAtIndex:2]]
			  type:@"text/html"
			  name:nil];
	  return YES;
	}
      else if ([queryPathComponents count] == 4) // we have /src/dsthost/client
	{
	  [response setContent:[self createPageForDestinationHost:
				[queryPathComponents objectAtIndex:2]
						      andClientIP:
				[queryPathComponents objectAtIndex:3]]
		          type:@"text/html"
		          name:nil];
	  return YES;
	}
    }
  else if ([[queryPathComponents objectAtIndex:1] isEqual:@"impersonator.css"])
    {
      [response setContent:[NSString stringWithContentsOfFile:
			[[NSBundle mainBundle] pathForResource:@"impersonator"
							ofType:@"css"]]
		      type:@"text/css"
		      name:nil];
      return YES;
    }
  return YES;
}
- (void) webAlert: (NSString*)message for: (WebServer*)http
{
  if (webServerHandlerDebug)
    NSLog(@"webAlert: %@", message);
}
- (void) webAudit: (NSString*)message for: (WebServer*)http
{
  if (webServerHandlerDebug)
    NSLog(@"webAudit: %@", message);
}
- (void) webLog: (NSString*)message for: (WebServer*)http
{
  if (webServerHandlerDebug)
    NSLog(@"webLog: %@", message);
}

@end
