/*
   Project: ImpersonatorToolKit

   Copyright (C) 2013 Sebastian Reitenbach

   Author: Sebastian Reitenbach

   Created: 2013-06-27 19:39:18 +0200 by sebastia

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import "Payload.h"
#import <GNUstepBase/GSMime.h>

static NSString *HTTP = @"HTTP/1";
static NSString *CRLF = @"\r\n";
static NSString *SPACE = @" ";
static NSString *DOT = @".";
static NSString *SEMICOLON = @";";
static NSString *EQUALSIGN = @"=";
static NSString *HEADEREND = @"\r\n\r\n";
static NSString *GET = @"GET";
static NSString *POST = @"POST";
static NSString *COOKIE = @"Cookie:";
static NSString *SETCOOKIE = @"Set-Cookie:";
static NSString *HOST = @"Host:";
static NSString *USERAGENT = @"User-Agent:";

@interface Payload(Private)
-(NSArray *) readHeaderLineFromPayloadString:(NSString *)_payloadString;
-(BOOL)parseConnection:(Connection *)_connection withPayload:(NSData *)_payload;
@end

@implementation Payload(Private)
-(NSArray *) readHeaderLineFromPayloadString:(NSString *)_payloadString
{
  NSArray *headerLines = nil;
  NSRange headerRange;

  // parse the content until two CRLFs are found
  headerRange = [_payloadString rangeOfString:HEADEREND];
  if (headerRange.location == NSNotFound)
    {
      return nil;
    }

  headerLines = [_payloadString componentsSeparatedByString:CRLF];
  
  return headerLines;
}
// returns YES, if found something worth storing, otherwise NO
-(BOOL)parseConnection:(Connection *)_connection withPayload:(NSData *)_payload
{
return NO;
  GSMimeParser *contentParser;
  GSMimeDocument *contentDocument;
  NSString *payloadString;

  payloadString = [[NSString alloc] initWithData:_payload encoding:NSASCIIStringEncoding];

  if (!([payloadString hasPrefix:GET] || [payloadString hasPrefix:POST]))
    {
      //[payloadString release];
      return NO;
    }

  contentParser = [GSMimeParser new];
  [contentParser setIsHttp];
  [contentParser setHeadersOnly];
  [contentParser setDefaultCharset: @"utf-8"];

  contentDocument = [contentParser mimeDocument];
//NSLog(@"payloadString: %@", payloadString);
//NSLog(@"here the document: %@", contentDocument);

  //[contentParser release];
  //[payloadString release];
  return YES;
}
@end

@implementation Payload

-(NSString *)clientIP
{
  return clientIP;
}
-(NSString *)serverIP
{
  return serverIP;
}
-(NSString *)urlPath
{
  return urlPath;
}
-(NSString *)host
{
  return host;
}
-(NSString *)domain
{
  return domain;
}
-(NSString *)userAgent
{
  return userAgent;
}
-(NSDictionary *)cookies
{
  return cookies;
}

-(void)dealloc
{
  [clientIP release];
  [serverIP release];
  [urlPath release];
  [host release];
  [domain release];
  [cookies release];
  [userAgent release];
  [super dealloc];
}

-(id)initWithConnection:(Connection *)_connection
{
  NSMutableData *cleanPayload;
  NSUInteger counter;
  NSString *payloadString;
  NSRange range;
  cleanPayload = [[NSMutableData alloc] init];
  NSArray *headers;

  self = [super init];
  if (self == nil)
    {
      RELEASE(cleanPayload);
      return nil;
    }

  payloadDebug = NO;

  for (counter =0;counter < [[_connection data] length];counter++)
    {
      NSData *subData = [[_connection data] subdataWithRange:NSMakeRange(counter,1)];
      const char *byte = [subData bytes];
      if (isprint(*byte) || *byte == 0xa || *byte == 0xd)
        {
          [cleanPayload appendData: subData];
        }
    }
  payloadString = [[NSString alloc] initWithData:cleanPayload encoding:NSASCIIStringEncoding];
  [self parseConnection:_connection withPayload:cleanPayload];
  [cleanPayload release];

  // check if it is a client request, whether there is a HTTP GET or POST request
  range = [payloadString rangeOfString:GET];
  if (range.location == NSNotFound || range.location > 5)
    range = [payloadString rangeOfString:POST];
  if (range.location == NSNotFound || range.location > 5)
    {
      if (payloadDebug)
        NSLog(@"did not found a GET or POST, must be an answer from the server");
      RELEASE(payloadString);
      return nil;
    }

  headers = [self readHeaderLineFromPayloadString: payloadString];
  [payloadString release];
  for (id header in headers)
    {
     // looking for the URL in GET or POST request
      range = [header rangeOfString:HTTP options:NSCaseInsensitiveSearch];
      if (range.location != NSNotFound)
        {
	  // check if the line containing HTTP/1 also starts with a GET or POST
	  // otherwise it might be something fishy
	  if (!([header hasPrefix:GET] || [header hasPrefix:POST]))
	    continue;
	  range = [header rangeOfString:SPACE];
	  // jump over the first space which is included within the range
          header = [header substringFromIndex:range.location + 1];
	  range = [header rangeOfString:SPACE];
          header = [header substringToIndex:range.location];
	  urlPath = header;
	  if (payloadDebug)
	    NSLog(@"examined URL path: %@", urlPath);
	  continue;
	}
      // looking for Host:
      range = [header rangeOfString:HOST options:NSCaseInsensitiveSearch];
      if (range.location != NSNotFound)
	{
	  range = [header rangeOfString:SPACE];
	  host = [[header substringFromIndex:range.location] 
			stringByTrimmingCharactersInSet:
			[NSCharacterSet whitespaceCharacterSet]];
	  range = [host rangeOfString:DOT];
	  if (range.location != NSNotFound)
	    {
	      if ([[host componentsSeparatedByString:DOT] count] < 3)
		{
	          ASSIGN(domain,host);
		}
	      else
		{
	          domain = [host substringFromIndex:range.location + 1];
		}
	    }
	  else
	    {
	      // we might have found an IPv6 address
	      ASSIGN(domain,host);
	    }
	  if (payloadDebug)
	    NSLog(@"found host: %@ and domain: %@", host, domain);
	  continue;
	}
      // check if it is a server to client connection
      range = [header rangeOfString:SETCOOKIE options:NSCaseInsensitiveSearch];
      if (range.location != NSNotFound)
	{
	  if (payloadDebug)
	    NSLog(@"detected a Set-Cookie server to client connection, skipping it");
	  return nil;
	}
	
      // save the user-agent:
      range = [header rangeOfString:USERAGENT options:NSCaseInsensitiveSearch];
      if (range.location != NSNotFound)
	{
	  range = [header rangeOfString:SPACE];
	  if (range.location != NSNotFound)
	    {
	      // jump over the leading space
	      userAgent = [header substringFromIndex:range.location +1];
	      if (payloadDebug)
	        NSLog(@"found User-Agent: %@", userAgent);
	    }
	  continue;
	}
      // looking for Cookie:
      range = [header rangeOfString:COOKIE options:NSCaseInsensitiveSearch];
      if (range.location != NSNotFound)
	{
	  // there can be multiple cookies sent to the server
	  // multiple cookies are separated by semicolons
	  NSString *cookieString;
	  NSArray *cookieArray;
	  range = [header rangeOfString:SPACE];
	  // Have seen Cookie: headers without any cookie
	  if (range.location == NSNotFound)
	    continue;
	  cookieString = [[header substringFromIndex:range.location] 
		stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceCharacterSet]];
	  cookieArray = [cookieString componentsSeparatedByString:SEMICOLON];
	  cookies = [[NSMutableDictionary alloc] init];
	  for (id cookie in cookieArray)
	    {
	      NSString *cookieName;
	      NSString *cookieValue;
	      
	      range = [cookie rangeOfString:EQUALSIGN];
	      // there may be spaces at the end of the cookie string, jump over it
	      if (range.location == NSNotFound)
		continue;
              cookieName = [[cookie substringToIndex:range.location]
                stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
	      // range.location + 1, otherwise it would include the equal sign
	      cookieValue = [cookie substringFromIndex:range.location+1];
	      [cookies setObject:cookieValue forKey:cookieName];
	    }
	  continue;
	}
    }
  clientIP = [_connection srcIP];
  serverIP = [_connection dstIP];
  return self;
}
@end
